/*
   Button-Numbers:
*/

const byte SoftButton1 = 0x00;
const byte SoftButton2 = 0x10;
const byte SoftButton3 = 0x20;
const byte SoftButton4 = 0x30;
const byte CalContinue = 0xC0;

String SoftButton1Txt;  // Text for one of the side-buttons - Max Length: 8 characters
String SoftButton2Txt;
String SoftButton3Txt;
String SoftButton4Txt;
String SoftButton5Txt;

int DisplayTimer = 0;
int DisplayTimeoutShort = 5000; // Milliseconds until Timeout
int DisplayTimeoutLong = 15000; // Milliseconds until Timeout

#define DISPLAY_DEBUG

void UpdateDisplay() {
  switch (SysStatus) {
    case STATE_NORM_OP:
      DisplayTimer = 0;
      DisplaySysMode(F("MODE_SELECT 1"), BLUE);
      SoftButton1Txt = F("  FREQ  ");
      SoftButton2Txt = F("   AMP  ");
      SoftButton3Txt = F("   MOD  ");
      SoftButton4Txt = F("SETTINGS");
      SoftButton5Txt = F("  MORE  ");
      DrawFrequencyArea(Freq);
      DrawAmplitudeArea(OUTPUT_dBm);
      DrawModulationArea(ModPara1, ModPara2, ModPara3, ModPara4);
      break;
    case STATE_MODE_SELECT_2:
      DisplayTimer = 0;
      DisplaySysMode("MODE_SELECT 1", BLUE);
      SoftButton1Txt = "  BURST ";
      SoftButton2Txt = "   RAMP ";
      SoftButton3Txt = "ADV_MOD ";
      SoftButton4Txt = "ARB_WAV ";
      SoftButton5Txt = "  BACK  ";
      DrawFrequencyArea(Freq);
      DrawAmplitudeArea(OUTPUT_dBm);
      DrawModulationArea(ModPara1, ModPara2, ModPara3, ModPara4);
      break;
    case STATE_ADV_MODULATION:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode(F("ADVANCED MODULATION-FUNCTIONS"), BLUE);
      Message(F("ERROR! Function not implemented yet"), AMBER);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      if ((DisplayTimer + DisplayTimeoutShort) >= millis()) {
        SysStatus = STATE_MODE_SELECT_2;
        DisplayTimer = 0;
      }
      break;
    case STATE_ARBITRARY_WAVE:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode(F("ARBITRARY WAVEFORM-FUNCTIONS"), BLUE);
      Message(F("ERROR! Function not implemented yet"), AMBER);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      DrawFrequencyArea(Freq);
      DrawAmplitudeArea(OUTPUT_dBm);
      DrawModulationArea(ModPara1, ModPara2, ModPara3, ModPara4);
      if ((DisplayTimer + DisplayTimeoutShort) >= millis()) {
        SysStatus = STATE_MODE_SELECT_2;
        DisplayTimer = 0;
      }
      break;
    case STATE_SIGPATH_CTRL:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode(F("SignalPath-Control"), BLUE);
      SoftButton1Txt = "FiltAuto";
      SoftButton2Txt = " Ext_Src";
      SoftButton3Txt = " Ext_AM ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      // drawXBitmap(DATA FOR THE BITMAP THAT SHOWS THE FILTER-SETUP HERE);
      DrawAmplitudeArea(OUTPUT_dBm);
      DrawModulationArea(ModPara1, ModPara2, ModPara3, ModPara4);
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_MODE_SELECT_2;
        DisplayTimer = 0;
      }
      break;
    case STATE_DEMO:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode(F("DEMO-MODE"), BLUE);
      Message(F("ERROR! Function not implemented yet"), AMBER);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      DrawFrequencyArea(Freq);
      DrawAmplitudeArea(OUTPUT_dBm);
      DrawModulationArea(ModPara1, ModPara2, ModPara3, ModPara4);
      if ((DisplayTimer + DisplayTimeoutShort) >= millis()) {
        SysStatus = STATE_MODE_SELECT_2;
        DisplayTimer = 0;
      }
      break;
    /*
     * *** CALIBRATION-SCREENS ***
    */
    case STATE_CALIBRATION:
      DisplayTimer = 0;
      DisplaySysMode(F("CALIBRATION-MODE"), CYAN);
      tft.drawRoundRect(10, 20, 160, 200, 4, AMBER);
      tft.setCursor(20, 30);
      tft.setTextColor(AMBER);
      tft.setTextSize(2);
      tft.print(F("Time since Startup:"));
      int16_t  x1, y1;
      uint16_t w, h;
      tft.getTextBounds(F("                "), 10, 70, &x1, &y1, &w, &h);
      tft.fillRect(x1, y1, w, h, BLACK);
      tft.print((millis() / 1000)); // Display time since Startup in Seconds
      tft.println(F(" Seconds"));
      SoftButton1Txt = "FREQ";
      SoftButton2Txt = "AMP";
      SoftButton3Txt = "MOD";
      SoftButton4Txt = "SETTINGS";
      SoftButton5Txt = "MORE";
      if (warmup == true) {
        tft.println(F("WARMING UP - PLEASE WAIT"));
      }
      else {
        tft.println(F("WARMUP COMPLETE"));
      }
      break;
    case STATE_SIGSOURCE_CAL:
      DisplayTimer = 0;
      break;
    case STATE_SIGPATH_CAL:
      DisplayTimer = 0;
      DisplaySysMode("SIGPATH-CALIBRATION", BLUE);
      // Put the SigPathCal-Screen here
      break;
    case STATE_SIGPATH_COMP:
      DisplayTimer = 0;
      DisplaySysMode("SIGPATH-CALIBRATION", BLUE);
      // Put the SigPathCal-Screen here
      break;
    case STATE_TOUCH_CAL:
      DisplayTimer = 0;
      DisplaySysMode("SIGPATH-CALIBRATION", BLUE);
      // Put the SigPathCal-Screen here
      break;

    /*
     * *** CONFIG-SCREENS ***
    */
    case STATE_CONFIG_1:
      DisplayTimer = 0;
      DisplaySysMode("CONFIG-PAGE 1", BLUE);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "  MORE  ";
      SoftButton5Txt = "  EXIT  ";
      // Put the SigPathCal-Screen here
      break;
    case STATE_CONFIG_2:
      DisplayTimer = 0;
      DisplaySysMode("CONFIG-PAGE 1", BLUE);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "  MORE  ";
      SoftButton5Txt = "  EXIT  ";
      // Put the SigPathCal-Screen here
      break;
    case STATE_CONFIG_3:
      DisplayTimer = 0;
      DisplaySysMode("CONFIG-PAGE 1", BLUE);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "  MORE  ";
      SoftButton5Txt = "  EXIT  ";
      // Put the SigPathCal-Screen here
      break;
    case STATE_CONFIG_4:
      DisplayTimer = 0;
      DisplaySysMode("CONFIG-PAGE 1", BLUE);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "  MORE  ";
      SoftButton5Txt = "  EXIT  ";
      // Put the SigPathCal-Screen here
      break;
    case STATE_CONFIG_5:
      DisplayTimer = 0;
      DisplaySysMode("CONFIG-PAGE 1", BLUE);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "  MORE  ";
      SoftButton5Txt = "  EXIT  ";
      // Put the SigPathCal-Screen here
      break;
    case STATE_CONFIG_6:
      DisplayTimer = 0;
      DisplaySysMode("CONFIG-PAGE 1", BLUE);
      SoftButton1Txt = "        ";
      SoftButton2Txt = "        ";
      SoftButton3Txt = "        ";
      SoftButton4Txt = "  MORE  ";
      SoftButton5Txt = "  EXIT  ";
      // Put the SigPathCal-Screen here
      break;

    /*
     * *** INPUT-SCREENS FOR DATA ***
    */
    case STATE_ENTER_FREQ:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("Enter Frequency!", BLUE);
      // Screen to enter the Frequency
      SoftButton1Txt = "   GHz";
      SoftButton2Txt = "   MHz";
      SoftButton3Txt = "   kHz";
      SoftButton4Txt = "    Hz";
      SoftButton5Txt = "  BACK";
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_NORM_OP;
        DisplayTimer = 0;
      }
      break;
    case STATE_ENTER_AMPLITUDE:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("Enter Amplitude!", BLUE);
      SoftButton1Txt = "  +dBm";
      SoftButton2Txt = "  -dBm";
      SoftButton3Txt = "  mVpp";
      SoftButton4Txt = "      ";
      SoftButton5Txt = "  BACK";
      // Screen to enter the Amplitude of the Signal
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_NORM_OP;
        DisplayTimer = 0;
      }
      break;
    case STATE_ENTER_MODULATION_TYPE:
      DisplayTimer = 0;
      DisplaySysMode("Select Modulation", BLUE);
      SoftButton1Txt = "   AM";
      SoftButton2Txt = "   FM";
      SoftButton3Txt = "   PM";
      SoftButton4Txt = "      ";
      SoftButton5Txt = "  BACK";
      // Put the Modulation-Select Screen here
      break;
    case STATE_ENTER_AM_PARAMETERS:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("AM-Parameter Setup", BLUE);
      SoftButton1Txt = "STRENGTH";
      SoftButton2Txt = "    kHz ";
      SoftButton3Txt = " digPat ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_ENTER_MODULATION_TYPE;
        DisplayTimer = 0;
      }
      // Put the AM-Settings here
      break;
    case STATE_ENTER_FM_PARAMETERS:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("FM-Parameter Setup", BLUE);
      SoftButton1Txt = "  WIDTH ";
      SoftButton2Txt = "    kHz ";
      SoftButton3Txt = " digPat ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_ENTER_MODULATION_TYPE;
        DisplayTimer = 0;
      }
      // Put the AM-Settings here
      break;
    case STATE_ENTER_PM_PARAMETERS:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("PM-Parameter Setup", BLUE);
      SoftButton1Txt = "  ANGLE ";
      SoftButton2Txt = "    kHz ";
      SoftButton3Txt = " digPat ";
      SoftButton4Txt = "        ";
      SoftButton5Txt = "  BACK  ";
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_ENTER_MODULATION_TYPE;
        DisplayTimer = 0;
      }
      // Put the AM-Settings here
      break;
    case STATE_ENTER_BURST:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("BURST-Parameter Setup", BLUE);
      SoftButton1Txt = " LENGTH ";
      SoftButton2Txt = "    kHz ";
      SoftButton3Txt = " digPat ";
      SoftButton4Txt = "    AMP ";
      SoftButton5Txt = "  BACK  ";
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_ADV_MODULATION;
        DisplayTimer = 0;
      }
      // Put the AM-Settings here
      break;
    case STATE_ENTER_RAMP:
      if (DisplayTimer == 0) {
        DisplayTimer = millis();
      }
      DisplaySysMode("RAMP-Parameter Setup", BLUE);
      SoftButton1Txt = " LENGTH ";
      SoftButton2Txt = "StartkHz";
      SoftButton3Txt = " StopkHz";
      SoftButton4Txt = " HighAMP";
      SoftButton5Txt = "  BACK  ";
      if ((DisplayTimer + DisplayTimeoutLong) >= millis()) {
        SysStatus = STATE_ADV_MODULATION;
        DisplayTimer = 0;
      }
      // Put the AM-Settings here
      break;
  }
#ifdef DISPLAY_DEBUG
  Message("TestMessage - This is where Errors or Warnings appear", RED);
#endif
  DrawButton(330, 30, SoftButton1Txt, false, AMBER, 2);
  DrawButton(330, 90, SoftButton2Txt, false, AMBER, 2);
  DrawButton(330, 150, SoftButton3Txt, false, AMBER, 2);
  DrawButton(330, 210, SoftButton4Txt, false, AMBER, 2);
  DrawButton(330, 270, SoftButton5Txt, false, AMBER, 2);
}

void UpdateDisplayPart(int dispPart) {
  switch (dispPart) {
    case INPUT_ENTER_FREQ:
      DrawFrequencyArea(keyboardInput);
      break;
  }
}

void UpdateButtons(bool pressed, byte button) {
  switch (button) {
    case SoftButton1:
      switch (pressed) {
        case true:
          DrawButton(165, 240, "CONTINUE", true, MAGENTA, 1);
          break;
        case false:

          break;
      }
      break;
    case CalContinue:
      switch (pressed) {
        case true:
          DrawButton(165, 240, "CONTINUE", true, MAGENTA, 1);
          break;
        case false:
          break;
      }
      break;
  }
}

/*
   This subprogram calls the graphics library to write a message into the message-area on the screen. The message Area has the coordinates: 140X - 470X / 0Y - 40Y.
   The Message Types are color-coded:
   - Normal System-Messages: WHITE
   - Warning-Messages: YELLOW
   - Error-Messages: RED
*/

void Message(String message, int mcolor) { // "message" is displayed in the color specified by "mcolor"
  tft.setCursor(140, 0);
  tft.setTextColor(mcolor);
  tft.setTextSize(1);
  tft.setTextWrap(false);
  tft.fillRect(140, 0, 340, 15, BLACK);
  tft.println(message);
}

void DisplaySysMode(String message, int mcolor) {

  tft.setCursor(0, 0);
  tft.setTextColor(mcolor);
  tft.setTextSize(1);
  tft.setTextWrap(false);
  tft.fillRect(0, 0, 130, 15, BLACK);
  tft.println(message);
}

/*
   This Subprogram draws the SoftButtons during normal Opration
   Library calls put here for reference:
   void drawRoundRect(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h, uint16_t radius, uint16_t color);
   void fillRoundRect(uint16_t x0, uint16_t y0, uint16_t w, uint16_t h, uint16_t radius, uint16_t color);

*/

void DrawButton (uint16_t XPosButt, uint16_t YPosButt, String ButtText, bool Highlight, uint16_t ButtColor, int TxtSize) {
  uint16_t TextX = XPosButt + 30;
  uint16_t TextY = YPosButt + 16;
  tft.setTextSize(2);
  tft.drawRoundRect(XPosButt, YPosButt, 150, 50, 5, ButtColor);
  XPosButt = XPosButt + 4;
  YPosButt = YPosButt + 3;
  switch (Highlight) {
    case false:
      tft.drawRoundRect(XPosButt, YPosButt, 142, 44, 4, ButtColor);
      tft.setCursor(TextX, TextY);
      tft.setTextColor(ButtColor);
      tft.println(ButtText);
      break;
    case true:
      tft.fillRoundRect(XPosButt, YPosButt, 142, 44, 4, ButtColor);
      tft.setCursor(TextX, TextY);
      tft.setTextColor(BLACK);
      tft.println(ButtText);
      break;
  }
}

void DrawFrequencyArea(unsigned long dispData) {
  if (UpdateFreqArea == true) {
    tft.drawRoundRect(2, 30, 290, 80, 2, AMBER);
    tft.setTextSize(2);
    float DispFrequency = dispData / 1000000.000;
    tft.setCursor(10, 40);
    tft.setTextColor(AMBER, BLACK);
    tft.println("Frequency");
    //  tft.println(" ");
    tft.setCursor(10, 70);
    int16_t  x1, y1;
    uint16_t w, h;
    tft.getTextBounds(F("               "), 10, 70, &x1, &y1, &w, &h);
    tft.fillRect(x1, y1, w, h, BLACK);

    tft.setTextSize(3);
#ifdef DISPLAY_DEBUG
    Serial.print(F(" DispFreq = "));
    Serial.println(DispFrequency);
#endif
    tft.print(DispFrequency, 6);
    tft.println(" MHZ");
    UpdateFreqArea = false;
  }
}

void DrawAmplitudeArea(unsigned long dispData) {
  if (UpdateAmpArea == true) {
    tft.drawRoundRect(2, 115, 290, 80, 2, AMBER);
    tft.setTextSize(2);
    float DispAmplitude = dispData;
    tft.setCursor(10, 130);
    tft.setTextColor(AMBER, BLACK);
    tft.println("Amplitude");
    //  tft.println(" ");
    tft.setCursor(10, 160);
#ifdef DISPLAY_DEBUG

#endif
    switch (AmplitudeModeDB) {
      case true:
        tft.print((DispAmplitude - OUTPUT_DBM_OFFSET), 1);
        tft.println(" dBm");
        break;
      case false:
        DispAmplitude = DispAmplitude - OUTPUT_DBM_OFFSET;
        float DispAmplitude_mV = pow(10, ((DispAmplitude - 10) / 20));
        tft.print(DispAmplitude_mV, 6);
        // Put Code here to convert Amplitude dB into mV
        tft.println(" mVpp");
        break;
    }
  }
  UpdateAmpArea = false;
}

void DrawModulationArea(unsigned long disData1, unsigned long dispData2, unsigned long dispData3, unsigned long dispData4) {
  tft.drawRoundRect(2, 200, 290, 120, 2, AMBER);
  if (UpdateModArea == true) {
    tft.fillRect(3, 201, 288, 118, BLACK);
    tft.setTextSize(2);
    tft.setCursor(10, 210);
    tft.setTextColor(AMBER, BLACK);
    tft.println("Modulation");
    tft.setCursor(10, 240);
#ifdef DISPLAY_DEBUG
#endif
  }
  UpdateModArea = false; // Modulation-Area updated, no further Updates needed until something changes (UpdateModArea set to True by different function)
}

