int PWR_SET; // 12bit-Value with a Vref of 5V to control the DAC that sets the AGC
int AGC_ATTEN_BIAS; // 12bit-Value with a Vref of 5V to control the DAC that sets the AGC-BIAS
int AD9910_AMP_OFFSET;

//RF_Level_Sense-Voltages @ Bias-Current = 1mA. Vf = 250mV:
//NoSignal: 300mV
//-12dBm FILTERED_RF-Power = 863mV
//-2dBm FILTERED_RF-Power = 2082mV
int RF_LEVEL_SENSE;
//RF_OUT_Sense-Voltages @ Bias-Current = 1mA. Vf = 250mV:
//NoSignal: 300mV
//0dBm Output-Power = 363mV
//+15dBm Output-Power = 656mV
int RF_OUT_SENSE;

volatile byte SIGPATH_CTRL = B00100101; // Controls the Output of the GPIO-Expander on the Analog board.
//BITMAP: UNUSED | UNUSED | Select 750MHz LPF | Select 1,5GHz LPF | Select Bypass-Path | Select Filtered Path | Select other signal source | Select ADF4351-Module

// Function to calculate the Attenuator and DAC-Settings to set the amplitude. Amplitude is expressed as (dBm + 110) x 10 (avoid Floating-Point calculations)
void SetAmplitude(int Amplitude) {
  byte Atten = Amplitude / 10;
  Amplitude = Amplitude + 10; // Compensation for the 10dB Loss in the Attenuator
  byte AmplitudeFine = Amplitude - Atten;
  SetAttenuator(Atten);
  int AttenBiasCalDataLocation = ((Freq / 25000000UL) * 2) + 0x0F0;
  int PwrOffsetCalDataLocation = (Freq / 25000000UL) * 2;
  int DAC_OFFSET;
  if (Freq >= 400000000UL) {
    if (Freq >= 3000000000UL) {
      AGC_ATTEN_BIAS = 0xFFF;
      DAC_OFFSET = ReadNVData(0x0F0);
    }
    else {
      AGC_ATTEN_BIAS = ReadNVData(AttenBiasCalDataLocation);
      DAC_OFFSET = ReadNVData(PwrOffsetCalDataLocation);
    }
    PWR_SET = AmplitudeFine + DAC_OFFSET; // PWR_SET is set to the Correct Value-Value corresponding to the currently set frequency.
    byte buf[2];
    buf[1] = PWR_SET;
    buf[0]  = PWR_SET >> 8;
    // FastWrite-Command to set DAC: Bit 15, 14 = 0 (Fast Write), Bit 13, 12 = 0 (DAC Enabled), Bit 11 - 0 = 12bit DAC-Value
    Wire.beginTransmission(PWR_SET_DAC);
    Wire.write(buf[0]);
    Wire.write(buf[1]);
    Wire.endTransmission();

    buf[1] = AGC_ATTEN_BIAS;
    buf[0]  = AGC_ATTEN_BIAS >> 8;
    // FastWrite-Command to set DAC: Bit 15, 14 = 0 (Fast Write), Bit 13, 12 = 0 (DAC Enabled), Bit 11 - 0 = 12bit DAC-Value
    Wire.beginTransmission(AGC_BIAS_DAC);
    Wire.write(buf[0]);
    Wire.write(buf[1]);
    Wire.endTransmission();
  }
  else {

  }
}

// Bitmap for the Attenuator: Latch_Enable(LE) | 32dB | 32dB | 16dB | 8dB | 4dB | 2dB | 1dB
void SetAttenuator(byte Attenuation) {
  if (Attenuation > 63) {
    Attenuation = Attenuation - 32;
    bitSet(Attenuation, 6);
  }
  bitClear(Attenuation, 7); // Make sure the LE-Pin is LOW
  Wire.beginTransmission(ATTEN_ADR);
  Wire.write(Attenuation);
  Wire.endTransmission();
  bitSet(Attenuation, 7); // Set LE-Pin HIGH
  Wire.beginTransmission(ATTEN_ADR);
  Wire.write(Attenuation);
  Wire.endTransmission();
  bitClear(Attenuation, 7); // Set LE-Pin LOW again
  Wire.beginTransmission(ATTEN_ADR);
  Wire.write(Attenuation);
  Wire.endTransmission();
}

void SetAGC_Bias() {
  byte buf[2];
  buf[1] = AGC_ATTEN_BIAS;
  buf[0]  = AGC_ATTEN_BIAS >> 8;
  Wire.beginTransmission(AGC_BIAS_DAC);
  Wire.write(buf[0]);
  Wire.write(buf[1]);
  Wire.endTransmission();
}

// Select the PLL or the AD9910 as the signal source
void SetSignalSource(bool SelectAD9910) {
  switch (SelectAD9910) {
    case true:
      bitSet(SIGPATH_CTRL, 1);
      bitClear(SIGPATH_CTRL, 0);
      Wire.beginTransmission(ANALOG_ADR);
      Wire.write(SIGPATH_CTRL);
      Wire.endTransmission();
      break;
    case false:
      bitSet(SIGPATH_CTRL, 0);
      bitClear(SIGPATH_CTRL, 1);
      Wire.beginTransmission(ANALOG_ADR);
      Wire.write(SIGPATH_CTRL);
      Wire.endTransmission();
      break;
  }

}

void SetLPF(int LPF_SEL) {
  switch (LPF_SEL) {
    case BYPASS:
      bitClear(SIGPATH_CTRL, 2);  // DISABLE the Filtered-Path
      bitSet(SIGPATH_CTRL, 3);  // ENABLE Bypass-Path
      bitClear(SIGPATH_CTRL, 4); // DISABLE 1.5GHz-Path to have that signal-path in a known state
      bitSet(SIGPATH_CTRL, 5); // ENABLE 750MHz-Path to have that signal-path in a known state
      break;
    case LPF_750:
      bitSet(SIGPATH_CTRL, 2);
      bitClear(SIGPATH_CTRL, 3);
      bitSet(SIGPATH_CTRL, 4);
      bitClear(SIGPATH_CTRL, 5);
      break;
    case LPF_1500:
      bitSet(SIGPATH_CTRL, 2);
      bitClear(SIGPATH_CTRL, 3);
      bitClear(SIGPATH_CTRL, 4);
      bitSet(SIGPATH_CTRL, 5);
      break;
    default:
      ErrorMode(DATA_ERROR_SIGPATH, true);
      break;
  }
  Wire.beginTransmission(ANALOG_ADR);
  Wire.write(SIGPATH_CTRL);
  Wire.endTransmission();
}

int ADC_READ(byte ADC_CHANNEL) { // Returns the voltage present at the selected input of the ADC in millivolt
  switch (ADC_CHANNEL) {
    case ADC_RF_OUT_SENSE:
      ads.setGain(GAIN_TWO);
      break;
    case ADC_RF_LEVEL_SENSE:
      ads.setGain(GAIN_ONE);
  }
  int ADC_Value = ads.readADC_SingleEnded(ADC_CHANNEL);
  int ADC_Volt = 0;
  switch (ADC_CHANNEL) {
    case ADC_RF_OUT_SENSE:
      ADC_Volt = ADC_Value * 1; // Scale the ADC-Value to the Vref and the selected gain (Gain = 1): 1bit = 2mV
      break;
    case ADC_RF_LEVEL_SENSE:
      ADC_Volt = ADC_Value * 2; // Scale the ADC-Value to the Vref and the selected gain (Gain = 1): 1bit = 2mV
      break;
  }
  return ADC_Value;
}

