// *** DISPLAY-TESTS ***
// *** Copied from the GraphicsTest-Example by Adafruit ***
void TestDisplay() {
  // read diagnostics (optional but can help debug problems)
  uint8_t x = tft.readcommand8(HX8357_RDPOWMODE);
  Serial.print(F("Display Power Mode: 0x")); Serial.println(x, HEX);
  x = tft.readcommand8(HX8357_RDMADCTL);
  Serial.print(F("MADCTL Mode: 0x")); Serial.println(x, HEX);
  x = tft.readcommand8(HX8357_RDCOLMOD);
  Serial.print(F("Pixel Format: 0x")); Serial.println(x, HEX);
  x = tft.readcommand8(HX8357_RDDIM);
  Serial.print(F("Image Format: 0x")); Serial.println(x, HEX);
  x = tft.readcommand8(HX8357_RDDSDR);
  Serial.print(F("Self Diagnostic: 0x")); Serial.println(x, HEX);
  // Test Display with Text and record the time
  Serial.print(F("Text                     "));
  Serial.println(testText());
  yield();

  Serial.print(F("FastLines              "));
  Serial.println(testFastLines(GREEN, RED));
  yield();
}

unsigned long testText() {
  tft.fillScreen(HX8357_BLACK);
  unsigned long start = micros();
  tft.setCursor(0, 0);
  tft.setTextColor(HX8357_WHITE);  tft.setTextSize(1);
  tft.println("Hello World!");
  tft.setTextColor(HX8357_YELLOW); tft.setTextSize(2);
  tft.println(1234.56);
  tft.setTextColor(HX8357_RED);    tft.setTextSize(3);
  tft.println(0xDEADBEEF, HEX);
  tft.println();
  tft.setTextColor(HX8357_GREEN);
  tft.setTextSize(5);
  tft.println("Groop");
  tft.setTextSize(2);
  tft.println(F("I implore thee,"));
  tft.setTextSize(1);
  tft.println(F("my foonting turlingdromes."));
  tft.println(F("And hooptiously drangle me"));
  tft.println(F("with crinkly bindlewurdles,"));
  tft.println(F("Or I will rend thee"));
  tft.println(F("in the gobberwarts"));
  tft.println(F("with my blurglecruncheon,"));
  tft.println(F("see if I don't!"));

  tft.setTextColor(HX8357_WHITE);
  tft.println(F("Alice was beginning to get very tired of sitting by her sister on the bank, and of having nothing to do: once or twice she had peeped into the book her sister was reading, but it had no pictures or conversations in it, 'and what is the use of a book,' thought Alice 'without pictures or conversations?'"));

  tft.println(F("So she was considering in her own mind (as well as she could, for the hot day made her feel very sleepy and stupid), whether the pleasure of making a daisy-chain would be worth the trouble of getting up and picking the daisies, when suddenly a White Rabbit with pink eyes ran close by her."));

  tft.println(F("There was nothing so very remarkable in that; nor did Alice think it so very much out of the way to hear the Rabbit say to itself, 'Oh dear! Oh dear! I shall be late!' (when she thought it over afterwards, it occurred to her that she ought to have wondered at this, but at the time it all seemed quite natural); but when the Rabbit actually took a watch out of its waistcoat-pocket, and looked at it, and then hurried on, Alice started to her feet, for it flashed across her mind that she had never before seen a rabbit with either a waistcoat-pocket, or a watch to take out of it, and burning with curiosity, she ran across the field after it, and fortunately was just in time to see it pop down a large rabbit-hole under the hedge."));

  tft.println(F("In another moment down went Alice after it, never once considering how in the world she was to get out again."));

  tft.println(F("The rabbit-hole went straight on like a tunnel for some way, and then dipped suddenly down, so suddenly that Alice had not a moment to think about stopping herself before she found herself falling down a very deep well."));

  tft.println(F("Either the well was very deep, or she fell very slowly, for she had plenty of time as she went down to look about her and to wonder what was going to happen next. First, she tried to look down and make out what she was coming to, but it was too dark to see anything; then she looked at the sides of the well, and noticed that they were filled with cupboards and book-shelves; here and there she saw maps and pictures hung upon pegs. She took down a jar from one of the shelves as she passed; it was labelled 'ORANGE MARMALADE', but to her great disappointment it was empty: she did not like to drop the jar for fear of killing somebody, so managed to put it into one of the cupboards as she fell past it."));

  return micros() - start;
}

unsigned long testFastLines(uint16_t color1, uint16_t color2) {
  unsigned long start;
  int           x, y, w = tft.width(), h = tft.height();

  tft.fillScreen(HX8357_BLACK);
  start = micros();
  for (y = 0; y < h; y += 5) tft.drawFastHLine(0, y, w, color1);
  for (x = 0; x < w; x += 5) tft.drawFastVLine(x, 0, h, color2);

  return micros() - start;
}

// Tune the Signal Sources from 1 to 3000MHz in 200MHz steps and measure amplitude - Compare with saved values. If values are out of tolerance set the Uncal-Bit
void SignalPathTest() {
  Serial.println(F("SignalPath-Test"));
  Serial.println(F("Setup ADF9910 in TEST-MODE"));
  SetupAD9910(STATE_TEST);
  // Put code to check for amplitudes here - ATTENTION! Use of "delay(0)" necessary to prevent problems with Wifi-Module, if ESP8266 is used!
  Serial.println(F("LF-Test done. Setting AD9910 to Standby-Mode"));
  SetupAD9910(STATE_STANDBY);
  Serial.println(F("ADF4351 in TEST-MODE"));
  SetupADF4351(STATE_TEST);
  Serial.println(F("ADF4351 STANDBY"));
  SetupADF4351(STATE_STANDBY);
  Serial.println(F("ADF4351 TEST COMPLETE"));
}

// Runs the SignalPath-Compensation
void SigPathCompensate() {
  SetAmplitude(115); // Set the Amplitude to +5dBm
  SetFreq(9000UL); // Set Frequency to 9kHz (Lowest frequency)
  AD9910_AMP_OFFSET = 0;
  int TestAmpAD9910 = 50;
  SetAmplitudeAD9910(TestAmpAD9910, false); // Set Amplitude of AD9910 to 50mVpp
  while (ADC_READ(ADC_RF_OUT_SENSE) <= 636) {
    TestAmpAD9910++;  // Increase Amplitude by 1
    SetAmplitudeAD9910(TestAmpAD9910, false); // Set AMplitude of AD9910
  }
  for (byte i = 0; i < 15; i++) {

  }
  SetFreq(400000000UL);
  PWR_SET = 0;
  while (ADC_READ(ADC_RF_OUT_SENSE) <= 636) {
    PWR_SET++;

  }
}

